@@ -5,7 +5,7 @@
 ;; Author: Oscar Figueiredo <oscar@xemacs.org>
 ;; Maintainer: Oscar Figueiredo <oscar@xemacs.org>
 ;; Created: Feb 1998
-;; Version: $Revision: 1.17 $
+;; Version: $Revision: 1.19 $
 ;; Keywords: help
 
 ;; This file is part of XEmacs
@@ -26,16 +26,25 @@
 ;; Boston, MA 02111-1307, USA.
 
 ;;; Commentary:
-;;    This package provides a common interface to query directory
-;;    servers using different protocols such as LDAP, CCSO PH/QI or BBDB.
-;;    Queries can be made through an interactive form. 
-;;    Inline query strings in buffers can also be expanded with appropriately
-;;    formatted query results (especially used to expand email addresses in
-;;    message buffers).  It also interfaces with the BBDB package to let you
-;;    register query results into your own BBDB database.
+;;    This package provides a common interface to query directory servers using
+;;    different protocols such as LDAP, CCSO PH/QI or BBDB.  Queries can be
+;;    made through an interactive form or inline. Inline query strings in
+;;    buffers are expanded with appropriately formatted query results
+;;    (especially used to expand email addresses in message buffers).  EUDC
+;;    also interfaces with the BBDB package to let you register query results
+;;    into your own BBDB database.
 
 ;;; Usage:
-;;    See the corresponding info file
+;;    EUDC comes with an extensive documentation, please refer to it.
+;;
+;;    The main entry points of EUDC are:
+;;      `eudc-query-form': Query a directory server from a query form
+;;      `eudc-expand-inline': Query a directory server for the e-mail address
+;;                            of the name before cursor and insert it in the 
+;;                            buffer
+;;      `eudc-get-phone': Get a phone number from a directory server
+;;      `eudc-get-email': Get an e-mail address from a directory server
+;;      `eudc-customize': Customize various aspects of EUDC
 
 ;;; Code:
 
@@ -49,7 +58,7 @@
 
 (autoload 'custom-menu-create "cus-edit")
 
-(require 'eudc-custom-vars)
+(require 'eudc-vars)
 
 
 
@@ -114,28 +123,44 @@
 (defun eudc-cdaar (obj)
   (cdr (car (car obj))))
 
-(if (not (fboundp 'plist-member))
-    (progn
-      (defun plist-member (plist prop)
-	(if (not (= 0 (% (length plist) 2)))
-	    (error "Malformed plist"))
-	(catch 'found
-	  (while plist
-	    (if (eq prop (car plist))
-		(throw 'found t))
-	    (setq plist (cdr (cdr plist))))
-	  nil))))
+(defun eudc-plist-member (plist prop)
+  "Return t if PROP has a value specified in PLIST."
+  (if (not (= 0 (% (length plist) 2)))
+      (error "Malformed plist"))
+  (catch 'found
+    (while plist
+      (if (eq prop (car plist))
+	  (throw 'found t))
+      (setq plist (cdr (cdr plist))))
+    nil))
 
-(if (not (fboundp 'lax-plist-get))
-    (defun lax-plist-get (plist prop &optional default)
-      (if (not (= 0 (% (length plist) 2)))
-	  (error "Malformed plist"))
-      (catch 'found
-	(while plist
-	  (if (equal prop (car plist))
-	      (throw 'found (car (cdr plist))))
-	  (setq plist (cdr (cdr plist))))
-	default)))
+;; Emacs' plist-get lacks third parameter
+(defun eudc-plist-get (plist prop &optional default)
+  "Extract a value from a property list.
+PLIST is a property list, which is a list of the form
+(PROP1 VALUE1 PROP2 VALUE2...).  This function returns the value
+corresponding to the given PROP, or DEFAULT if PROP is not
+one of the properties on the list."
+  (if (eudc-plist-member plist prop)
+      (plist-get plist prop)
+    default))
+
+(defun eudc-lax-plist-get (plist prop &optional default)
+  "Extract a value from a lax property list.
+
+LAX-PLIST is a lax property list, which is a list of the form (PROP1
+VALUE1 PROP2 VALUE2...), where comparions between properties is done
+using `equal' instead of `eq'.  This function returns the value
+corresponding to the given PROP, or DEFAULT if PROP is not one of the
+properties on the list."
+  (if (not (= 0 (% (length plist) 2)))
+      (error "Malformed plist"))
+  (catch 'found
+    (while plist
+      (if (equal prop (car plist))
+	  (throw 'found (car (cdr plist))))
+      (setq plist (cdr (cdr plist))))
+    default))
 
 (if (not (fboundp 'split-string))
     (defun split-string (string &optional pattern)
@@ -156,12 +181,20 @@ If PATTERN is omitted, it defaults to \"[ \\f\\t\\n\\r\\v]+\"."
 		  (cons (substring string start) parts)
 		parts)))))
 
-(if eudc-xemacs-p
-    (defalias 'eudc-plist-get 'plist-get)
-  (defun eudc-plist-get (plist prop &optional default)
-    (if (plist-member plist prop)
-	(plist-get plist prop)
-      default)))
+(defun eudc-replace-in-string (str regexp newtext)
+  "Replace all matches in STR for REGEXP with NEWTEXT string,
+ and returns the new string."
+  (let ((rtn-str "")
+	(start 0)
+	match prev-start)
+    (while (setq match (string-match regexp str start))
+      (setq prev-start start
+	    start (match-end 0)
+	    rtn-str
+	    (concat rtn-str
+		    (substring str prev-start match)
+		    newtext)))
+    (concat rtn-str (substring str start))))
 
 ;;}}} 
 
@@ -169,11 +202,11 @@ If PATTERN is omitted, it defaults to \"[ \\f\\t\\n\\r\\v]+\"."
 
 (defun eudc-server-local-variable-p (var)
   "Return non-nil if variable has server local bindings"
-  (plist-member (get var 'eudc-locals) 'server))
+  (eudc-plist-member (get var 'eudc-locals) 'server))
 
 (defun eudc-protocol-local-variable-p (var)
   "Return non-nil if variable has protocol local bindings"
-  (plist-member (get var 'eudc-locals) 'protocol))
+  (eudc-plist-member (get var 'eudc-locals) 'protocol))
 
 (defun eudc-default-set (var val)
   "Set the EUDC default value of VAR to VAL.
@@ -246,12 +279,12 @@ PROTOCOL defaults to `eudc-protocol'"
 	 protocol-locals)
     (if (not (and  (boundp var)
 		   eudc-locals
-		   (plist-member eudc-locals 'protocol)))
+		   (eudc-plist-member eudc-locals 'protocol)))
 	'unbound
       (setq protocol-locals (eudc-plist-get eudc-locals 'protocol))
-      (lax-plist-get protocol-locals 
-		     (or protocol
-			 eudc-protocol) 'unbound))))
+      (eudc-lax-plist-get protocol-locals 
+			  (or protocol
+			      eudc-protocol) 'unbound))))
 
 (defun eudc-variable-server-value (var &optional server)
   "Return the value of VAR local to SERVER.
@@ -261,12 +294,12 @@ SERVER defaults to `eudc-server'"
 	 server-locals)
     (if (not (and (boundp var)
 		  eudc-locals
-		  (plist-member eudc-locals 'server)))
+		  (eudc-plist-member eudc-locals 'server)))
 	'unbound
       (setq server-locals (eudc-plist-get eudc-locals 'server))
-      (lax-plist-get server-locals 
-		     (or server
-			 eudc-server) 'unbound))))
+      (eudc-lax-plist-get server-locals 
+			  (or server
+			      eudc-server) 'unbound))))
 
 (defun eudc-update-variable (var)
   "Set the value of VAR according to its locals.
@@ -361,7 +394,7 @@ accordingly. Otherwise it is set to its EUDC default binding"
 	eudc-insertion-marker nil))
 
 (defun eudc-query (query &optional return-attributes no-translation)
-   "Query the directory server with QUERY.
+   "Query the current directory server with QUERY.
 QUERY is a list of cons cells (ATTR . VALUE) where ATTR is an attribute
 name and VALUE the corresponding value.  
 If NO-TRANSLATION is non nil, ATTR is translated according to 
@@ -505,7 +538,8 @@ otherwise they are formatted according to `eudc-user-attribute-names-alist'."
 		   "Quit")
     (eudc-mode)
     (widget-setup)
-    (goto-char first-record)))
+    (if first-record
+	(goto-char first-record))))
 
 (defun eudc-process-form ()
   "Process the query form in current buffer and display the results."
@@ -522,8 +556,7 @@ otherwise they are formatted according to `eudc-user-attribute-names-alist'."
 					     query-alist)))))
 	      eudc-form-widget-list)
       (kill-buffer (current-buffer))
-      (eudc-display-records (eudc-query query-alist) eudc-use-raw-directory-names)
-      )))
+      (eudc-display-records (eudc-query query-alist) eudc-use-raw-directory-names))))
          
            
 
@@ -675,7 +708,7 @@ server for future sessions."
 						 eudc-known-protocols)))))
   (unless (or (member protocol
 		      eudc-supported-protocols)
-	      (load (concat "eudc-bck-" (symbol-name protocol)) t))
+	      (load (concat "eudcb-" (symbol-name protocol)) t))
     (error "Unsupported protocol: %s" protocol))
   (run-hooks 'eudc-switch-from-server-hook)
   (setq eudc-protocol protocol)
@@ -754,13 +787,13 @@ otherwise a list of symbols is returned."
 		  val (eudc-cdar query-alist)
 		  cell (assq key query))
 	    (if cell
-		(setcdr cell (concat val " " (cdr cell)))
+		(setcdr cell (concat (cdr cell) " " val))
 	      (setq query (cons (car query-alist) query)))
 	    (setq query-alist (cdr query-alist)))
 	  query)
       (if eudc-protocol-has-default-query-attributes
-	  (car words)
-	(list (cons 'name (car words)))))))
+	  (mapconcat 'identity words " ")
+	(list (cons 'name (mapconcat 'identity words " ")))))))
 
 (defun eudc-extract-n-word-formats (format-list n)
   "Extract a list of N-long formats from FORMAT-LIST.
@@ -810,7 +843,7 @@ see `eudc-inline-expansion-servers'"
 					'move)
 		    (goto-char (match-end 0)))
 		(point)))
-	 (query-words (split-string (buffer-substring beg end)"[ \t]+"))
+	 (query-words (split-string (buffer-substring beg end) "[ \t]+"))
 	 query-formats
 	 response
 	 response-string
@@ -833,9 +866,9 @@ see `eudc-inline-expansion-servers'"
 	   (t
 	    (error "Wrong value for `eudc-inline-expansion-servers': %S"
 		   eudc-inline-expansion-servers))))
-    (if eudc-max-servers-to-query
-	(setq servers (subseq servers 0 (min eudc-max-servers-to-query
-					     (length servers)))))
+    (if (and eudc-max-servers-to-query
+	     (> (length servers) eudc-max-servers-to-query))
+	(setcdr (nthcdr (1- eudc-max-servers-to-query) servers) nil))
 
     (condition-case signal
 	(progn
